@@ -56,7 +56,14 @@ class Utilities(web.Utilities):
         if not (obj and self.inDocumentContent(obj)):
             return super().isStaticTextLeaf(obj)
 
-        if obj.childCount:
+        try:
+            childCount = obj.childCount
+        except:
+            msg = "CHROMIUM: Exception getting child count of %s" % obj
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return False
+
+        if childCount:
             return False
 
         if self.isListItemMarker(obj):
@@ -101,12 +108,23 @@ class Utilities(web.Utilities):
 
         rv = False
         if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_ITEM:
-            rv = self._getTag(obj) in ["::marker", None] and obj.parent[0] == obj
+            tag = self._getTag(obj)
+            if tag == "::marker":
+                rv = True
+            elif tag is not None:
+                rv = False
+            elif obj.parent.childCount > 1:
+                rv = obj.parent[0] == obj
+            else:
+                rv = obj.name != self.displayedText(obj.parent)
 
         self._isListItemMarker[hash(obj)] = rv
         return rv
 
     def selectedChildCount(self, obj):
+        if not obj:
+            return []
+
         count = super().selectedChildCount(obj)
         if count or "Selection" in pyatspi.listInterfaces(obj):
             return count
@@ -123,24 +141,22 @@ class Utilities(web.Utilities):
         return count
 
     def selectedChildren(self, obj):
+        if not obj:
+            return []
+
         result = super().selectedChildren(obj)
-        if result or "Selection" in pyatspi.listInterfaces(obj):
-            return result
 
-        try:
-            childCount = obj.childCount
-        except:
-            msg = "CHROMIUM: Exception getting child count of %s" % obj
+        # The fix for this issue landed in 90.0.4413.0.
+        if obj.getRole() == pyatspi.ROLE_MENU and not self.inDocumentContent(obj) and len(result) > 1:
+            msg = "CHROMIUM: Browser menu %s claims more than one state-selected child." % obj
             debug.println(debug.LEVEL_INFO, msg, True)
-            return result
 
-        # HACK: Ideally, we'd use the selection interface to get the selected
-        # children. But that interface is not implemented yet. This hackaround
-        # is extremely non-performant.
-        for i in range(childCount):
-            child = obj[i]
-            if child and child.getState().contains(pyatspi.STATE_SELECTED):
-                result.append(child)
+            isFocused = lambda x: x and x.getState().contains(pyatspi.STATE_FOCUSED)
+            focused = list(filter(isFocused, result))
+            if len(focused) == 1:
+                msg = "CHROMIUM: Suspecting %s is the only actually-selected child" % focused[0]
+                debug.println(debug.LEVEL_INFO, msg, True)
+                return focused
 
         return result
 
@@ -299,6 +315,9 @@ class Utilities(web.Utilities):
         if obj.getRole() == pyatspi.ROLE_LIST_ITEM:
             listbox = listbox.parent
 
+        if not listbox:
+            return result
+
         # The listbox sometimes claims to be a redundant object rather than a listbox.
         # Clearing the AT-SPI2 cache seems to be the trigger.
         if not (listbox and listbox.getRole() in roles):
@@ -483,41 +502,58 @@ class Utilities(web.Utilities):
 
         return super().findAllDescendants(root, includeIf, excludeIf)
 
-    def _accessibleAtPoint(self, root, x, y, coordType=None):
-        if self.isHidden(root):
-            return None
-
-        try:
-            component = root.queryComponent()
-        except:
-            msg = "CHROMIUM: Exception querying component of %s" % root
-            debug.println(debug.LEVEL_INFO, msg, True)
-            return None
-
-        result = component.getAccessibleAtPoint(x, y, coordType)
+    def accessibleAtPoint(self, root, x, y, coordType=None):
+        result = super().accessibleAtPoint(root, x, y, coordType)
 
         # Chromium cannot do a hit test of web content synchronously. So what it
         # does is return a guess, then fire off an async hit test. The next time
         # one calls it, Chromium returns the previous async hit test result if
         # the point is still within its bounds. Therefore, we need to call
-        # getAccessibleAtPoint() twice to be safe.
-        result = component.getAccessibleAtPoint(x, y, coordType)
-
-        msg = "CHROMIUM: %s is descendant of %s at (%i, %i)" % (result, root, x, y)
+        # accessibleAtPoint() twice to be safe.
+        msg = "CHROMIUM: Getting accessibleAtPoint again due to async hit test result."
         debug.println(debug.LEVEL_INFO, msg, True)
+        result = super().accessibleAtPoint(root, x, y, coordType)
         return result
 
-    def descendantAtPoint(self, root, x, y, coordType=None):
-        if coordType is None:
-            coordType = pyatspi.DESKTOP_COORDS
+    def _isActiveAndShowingAndNotIconified(self, obj):
+        if super()._isActiveAndShowingAndNotIconified(obj):
+            return True
 
-        result = None
-        if self.isDocument(root):
-            result = self._accessibleAtPoint(root, x, y, coordType)
+        if obj and obj.getApplication() != self._script.app:
+            return False
 
-        root = result or root
-        result = super().descendantAtPoint(root, x, y, coordType)
-        if self.isListItemMarker(result) or self.isStaticTextLeaf(result):
-            return result.parent
+        # FIXME: This can potentially be non-performant because AT-SPI2 will recursively
+        # clear the cache of all descendants. This is an attempt to work around what may
+        # be a lack of window:activate and object:state-changed events from Chromium
+        # windows in at least some environments.
+        try:
+            msg = "CHROMIUM: Clearing cache for %s" % obj
+            debug.println(debug.LEVEL_INFO, msg, True)
+            obj.clearCache()
+        except:
+            msg = "CHROMIUM: Exception clearing cache for %s" % obj
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return False
 
-        return result
+        if super()._isActiveAndShowingAndNotIconified(obj):
+            msg = "CHROMIUM: %s deemed to be active and showing after cache clear" % obj
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return True
+
+        return False
+
+    def _shouldCalculatePositionAndSetSize(self, obj):
+        # Chromium has accessible menu items which are not focusable and therefore do not
+        # have a posinset and setsize calculated. But they may claim to be the selected
+        # item when an accessible child is selected (e.g. "zoom" when "+" or "-" gains focus.
+        # Normally we calculate posinset and setsize when the application hasn't provided it.
+        # We don't want to do that in the case of menu items like "zoom" because our result
+        # will not jibe with the values of its siblings. Thus if a sibling has a value,
+        # assume that the missing attributes are missing on purpose.
+        for sibling in obj.parent:
+            if self.getPositionInSet(sibling) is not None:
+                msg = "CHROMIUM: %s's sibling %s has posinset." % (obj, sibling)
+                debug.println(debug.LEVEL_INFO, msg, True)
+                return False
+
+        return True
