@@ -31,6 +31,7 @@ import pyatspi
 
 from orca import braille
 from orca import braille_generator
+from orca import debug
 from orca import messages
 from orca import object_properties
 from orca import orca_state
@@ -45,7 +46,7 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
         if not self._script.utilities.inDocumentContent(obj):
             return super().getLocalizedRoleName(obj, **args)
 
-        roledescription = self._script.utilities.getRoleDescription(obj)
+        roledescription = self._script.utilities.getRoleDescription(obj, True)
         if roledescription:
             return roledescription
 
@@ -54,7 +55,10 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
     def _generateRoleName(self, obj, **args):
         """Prevents some roles from being displayed."""
 
-        roledescription = self._script.utilities.getRoleDescription(obj)
+        if not self._script.utilities.inDocumentContent(obj):
+            return super()._generateRoleName(obj, **args)
+
+        roledescription = self._script.utilities.getRoleDescription(obj, True)
         if roledescription:
             return [roledescription]
 
@@ -62,6 +66,7 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
                         pyatspi.ROLE_PARAGRAPH,
                         pyatspi.ROLE_STATIC,
                         pyatspi.ROLE_SECTION,
+                        pyatspi.ROLE_REDUNDANT_OBJECT,
                         pyatspi.ROLE_UNKNOWN]
 
         state = obj.getState()
@@ -88,7 +93,11 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
                 result.append(messages.IMAGE_MAP_LINK)
 
         elif role not in doNotDisplay:
-            result = super()._generateRoleName(obj, **args)
+            label = self._script.utilities.labelForCellCoordinates(obj)
+            if label:
+                result.append(label)
+            else:
+                result = super()._generateRoleName(obj, **args)
 
         index = args.get('index', 0)
         total = args.get('total', 1)
@@ -102,15 +111,22 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
         return result
 
     def _generateLabelOrName(self, obj, **args):
+        if not self._script.utilities.inDocumentContent(obj):
+            return super()._generateLabelOrName(obj, **args)
+
         if self._script.utilities.isTextBlockElement(obj):
             return []
 
-        if self._script.utilities.inDocumentContent(obj) and obj.name:
-            return [obj.name]
+        if obj.getState().contains(pyatspi.STATE_EDITABLE) \
+           and self._script.utilities.isCodeDescendant(obj):
+            return []
 
-        return super()._generateLabelOrName(obj, **args)
+        return self._generateName(obj, **args)
 
     def _generateLabel(self, obj, **args):
+        if not self._script.utilities.inDocumentContent(obj):
+            return super()._generateLabel(obj, **args)
+
         label, objects = self._script.utilities.inferLabelFor(obj)
         if label:
             return [label]
@@ -118,6 +134,9 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
         return super()._generateLabel(obj, **args)
 
     def _generateLabelAndName(self, obj, **args):
+        if not self._script.utilities.inDocumentContent(obj):
+            return super()._generateLabelAndName(obj, **args)
+
         if self._script.utilities.isTextBlockElement(obj):
             return []
 
@@ -127,10 +146,46 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
 
         return super()._generateLabelAndName(obj, **args)
 
+    def _generateDescription(self, obj, **args):
+        if not self._script.utilities.inDocumentContent(obj):
+            return super()._generateDescription(obj, **args)
+
+        if self._script.utilities.preferDescriptionOverName(obj):
+            return []
+
+        return super()._generateDescription(obj, **args)
+
+    def _generateName(self, obj, **args):
+        if not self._script.utilities.inDocumentContent(obj):
+            return super()._generateName(obj, **args)
+
+        brailleLabel = self._script.utilities.objectAttributes(obj).get("braillelabel")
+        if brailleLabel:
+            return [brailleLabel]
+
+        if self._script.utilities.preferDescriptionOverName(obj):
+            return [obj.description]
+
+        if obj.name and not self._script.utilities.hasValidName(obj):
+            return []
+
+        result = super()._generateName(obj, **args)
+        if result and result[0] and not self._script.utilities.hasExplicitName(obj):
+            result[0] = result[0].strip()
+        elif not result and obj.getRole() == pyatspi.ROLE_CHECK_BOX:
+            gridCell = pyatspi.findAncestor(obj, self._script.utilities.isGridCell)
+            if gridCell:
+                return super()._generateName(gridCell, **args)
+
+        return result
+
     def _generateExpandedEOCs(self, obj, **args):
         """Returns the expanded embedded object characters for an object."""
-        result = []
 
+        if not self._script.utilities.inDocumentContent(obj):
+            return super()._generateExpandedEOCs(obj, **args)
+
+        result = []
         startOffset = args.get('startOffset', 0)
         endOffset = args.get('endOffset', -1)
         text = self._script.utilities.expandEOCs(obj, startOffset, endOffset)
@@ -139,7 +194,16 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
 
         return result
 
+    def _generateRealActiveDescendantDisplayedText(self, obj, **args):
+        if not self._script.utilities.inDocumentContent(obj):
+            return super()._generateRealActiveDescendantDisplayedText(obj, **args)
+
+        return self._generateDisplayedText(obj, **args)
+
     def _generateTableCellRow(self, obj, **args):
+        if not self._script.utilities.inDocumentContent(obj):
+            return super()._generateTableCellRow(obj, **args)
+
         if not self._script.inFocusMode():
             return super()._generateTableCellRow(obj, **args)
 
@@ -148,12 +212,20 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
 
         isRow = lambda x: x and x.getRole() == pyatspi.ROLE_TABLE_ROW
         row = pyatspi.findAncestor(obj, isRow)
-        if row and row.name:
+        if row and row.name and not self._script.utilities.isLayoutOnly(row):
             return self.generate(row, includeContext=False)
 
         return super()._generateTableCellRow(obj, **args)
 
     def generateBraille(self, obj, **args):
+        if not self._script.utilities.inDocumentContent(obj):
+            msg = "WEB: %s is not in document content. Calling default braille generator." % obj
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return super().generateBraille(obj, **args)
+
+        msg = "WEB: Generating braille for document object %s" % obj
+        debug.println(debug.LEVEL_INFO, msg, True)
+
         result = []
 
         args['includeContext'] = not self._script.utilities.inDocumentContent(obj)
@@ -165,6 +237,8 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
             oldRole = self._overrideRole(pyatspi.ROLE_STATIC, args)
         elif self._script.utilities.treatAsDiv(obj, offset=args.get('startOffset')):
             oldRole = self._overrideRole(pyatspi.ROLE_SECTION, args)
+        elif self._script.utilities.treatAsEntry(obj):
+            oldRole = self._overrideRole(pyatspi.ROLE_ENTRY, args)
 
         if obj.getRole() == pyatspi.ROLE_MENU_ITEM:
             comboBox = self._script.utilities.ancestorWithRole(
@@ -193,7 +267,7 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
             return []
 
         result = []
-        contents = self._script.utilities.filterContentsForPresentation(contents, False)
+        contents = self._script.utilities.filterContentsForPresentation(contents, True)
 
         obj, offset = self._script.utilities.getCaretContext(documentFrame=None)
         index = self._script.utilities.findObjectInContents(obj, offset, contents)
